﻿using Esri.ArcGISRuntime.Controls;
using Esri.ArcGISRuntime.Geometry;
using Esri.ArcGISRuntime.Layers;
using Esri.ArcGISRuntime.Symbology;
using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace BitmapLeak
{
    public partial class MainWindow : Window
    {
        private GraphicsLayer graphicsLayer = new GraphicsLayer();
        private Graphic graphic = new Graphic();
        private PictureMarkerSymbol pictureSymbol = new PictureMarkerSymbol();
        private DispatcherTimer timer = new DispatcherTimer();
        private int markerSize = 0;
        private int distance = 0;

        public MainWindow()
        {
            InitializeComponent();

            this.graphic.Symbol = this.pictureSymbol;
            this.graphic.Geometry = new MapPoint(0, 0, SpatialReferences.Wgs84);
            this.graphicsLayer.Graphics.Add(this.graphic);

            this.MyMapView.Map.Layers.Add(this.graphicsLayer);

            this.timer.Tick += onTimerTick;
            // This is a faster tick than my use case but it will exacerbate the problem.
            this.timer.Interval = TimeSpan.FromMilliseconds(1);

            this.timer.Start();
        }

        private void onTimerTick(object sender, EventArgs e)
        {
            // Render a WPF visual using RenderTargetBitmap.
            // This is intentionally un-optimized (re-newed every time) to simulate my use case.
            RenderTargetBitmap rtb = new RenderTargetBitmap(250, 250, 96, 96, System.Windows.Media.PixelFormats.Pbgra32);
            Test test = new Test();
            Size size = new Size(300, 300);
            test.Measure(size);
            test.Arrange(new Rect(size));
            rtb.Render(test);

            BitmapFrame frame = BitmapFrame.Create(rtb);
            PngBitmapEncoder encoder = new PngBitmapEncoder();
            encoder.Frames.Add(frame);

            byte[] array;
            using (MemoryStream stream = new MemoryStream())
            {
                encoder.Save(stream);
                array = stream.ToArray();
            }
            // Set the symbol source to the byte array, this seems to be the culprit.
            // It's also worth noting that using the PictureMarkerSymbol.SetSourcAsync(Stream) does the same thing.
            this.pictureSymbol.SetSource(array);

            // Reset the width/height properties. Probably unrelated?
            this.pictureSymbol.Width = markerSize;
            this.pictureSymbol.Height = markerSize;

            // Move the geometry around some. Also probably unrelated?
            MapPoint newGeom = new MapPoint(0 + distance * Math.Cos(0.5), 0 + distance * Math.Sin(0.5));
            this.graphic.Geometry = newGeom;
            this.graphic.Symbol = this.pictureSymbol;

            // Increment the property incrementers
            if (distance > 50)
                distance = 0;
            else
                distance += 5;

            if (markerSize <= 255)
                markerSize++;
            else
                markerSize = 0;
        }

        private void MyMapView_LayerLoaded(object sender, LayerLoadedEventArgs e)
        {
            if (e.LoadError == null)
                return;

            Debug.WriteLine(string.Format("Error while loading layer : {0} - {1}", e.Layer.ID, e.LoadError.Message));
        }
    }
}
